
//
//  PRotX86.cpp - Packed Rotation 
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 2 Jun 98 - James Leiterman
//
//  Last Modification
//
//      23 Jul 2001 - JCL - Brought up to date.
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include "CpuAsm.h"    
#include "vmp3D.h"
#include "vmp3DEx.h"
#include "PRot.h"
#include "PRotEx.h"



//
//  Setup Packed Rotation
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void PRotSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
        
    if ( CPUBITS_MMX & pcinfo->nBits )
      {
        vmp_psraB =             vmp_psraBAsmMMX;
        vmp_psraB0 =            vmp_psraBAsmMMX;
        vmp_psraH =             vmp_psraHAsmMMX;
        vmp_psraH0 =            vmp_psraHAsmMMX;
        vmp_psraW =             vmp_psraWAsmMMX;
        vmp_psraW0 =            vmp_psraWAsmMMX;

        vmp_psra1B =            vmp_psra1BAsmMMX;
        vmp_psra1B0 =           vmp_psra1BAsmMMX;
        vmp_psra1H =            vmp_psra1HAsmMMX;
        vmp_psra1H0 =           vmp_psra1HAsmMMX;
        vmp_psra1W =            vmp_psra1WAsmMMX;
        vmp_psra1W0 =           vmp_psra1WAsmMMX;

        vmp_psllB =             vmp_psllBAsmMMX;
        vmp_psllB0 =            vmp_psllBAsmMMX;
        vmp_psllH =             vmp_psllHAsmMMX;
        vmp_psllH0 =            vmp_psllHAsmMMX;
        vmp_psllW =             vmp_psllWAsmMMX;
        vmp_psllW0 =            vmp_psllWAsmMMX;

        vmp_psll1B =            vmp_psll1BAsmMMX;
        vmp_psll1B0 =           vmp_psll1BAsmMMX;
        vmp_psll1H =            vmp_psll1HAsmMMX;
        vmp_psll1H0 =           vmp_psll1HAsmMMX;
        vmp_psll1W =            vmp_psll1WAsmMMX;
        vmp_psll1W0 =           vmp_psll1WAsmMMX;

        vmp_psrlB =             vmp_psrlBAsmMMX;
        vmp_psrlB0 =            vmp_psrlBAsmMMX;
        vmp_psrlH =             vmp_psrlHAsmMMX;
        vmp_psrlH0 =            vmp_psrlHAsmMMX;
        vmp_psrlW =             vmp_psrlWAsmMMX;
        vmp_psrlW0 =            vmp_psrlWAsmMMX;

        vmp_psrl1B =            vmp_psrl1BAsmMMX;
        vmp_psrl1B0 =           vmp_psrl1BAsmMMX;
        vmp_psrl1H =            vmp_psrl1HAsmMMX;
        vmp_psrl1H0 =           vmp_psrl1HAsmMMX;
        vmp_psrl1W =            vmp_psrl1WAsmMMX;
        vmp_psrl1W0 =           vmp_psrl1WAsmMMX;

        vmp_prolB =             vmp_prolBAsmMMX;
        vmp_prolB0 =            vmp_prolBAsmMMX;
        vmp_prolH =             vmp_prolHAsmMMX;
        vmp_prolH0 =            vmp_prolHAsmMMX;
        vmp_prolW =             vmp_prolWAsmMMX;
        vmp_prolW0 =            vmp_prolWAsmMMX;

        vmp_prol1B =            vmp_prol1BAsmMMX;
        vmp_prol1B0 =           vmp_prol1BAsmMMX;
        vmp_prol1H =            vmp_prol1HAsmMMX;
        vmp_prol1H0 =           vmp_prol1HAsmMMX;
        vmp_prol1W =            vmp_prol1WAsmMMX;
        vmp_prol1W0 =           vmp_prol1WAsmMMX;
      }

    else
      {
        vmp_psraB =             vmp_psraBGeneric;
        vmp_psraB0 =            vmp_psraBGeneric;
        vmp_psraH =             vmp_psraHGeneric;
        vmp_psraH0 =            vmp_psraHGeneric;
        vmp_psraW =             vmp_psraWGeneric;
        vmp_psraW0 =            vmp_psraWGeneric;

        vmp_psra1B =            vmp_psra1BGeneric;
        vmp_psra1B0 =           vmp_psra1BGeneric;
        vmp_psra1H =            vmp_psra1HGeneric;
        vmp_psra1H0 =           vmp_psra1HGeneric;
        vmp_psra1W =            vmp_psra1WGeneric;
        vmp_psra1W0 =           vmp_psra1WGeneric;

        vmp_psllB =             vmp_psllBGeneric;
        vmp_psllB0 =            vmp_psllBGeneric;
        vmp_psllH =             vmp_psllHGeneric;
        vmp_psllH0 =            vmp_psllHGeneric;
        vmp_psllW =             vmp_psllWGeneric;
        vmp_psllW0 =            vmp_psllWGeneric;

        vmp_psll1B =            vmp_psll1BGeneric;
        vmp_psll1B0 =           vmp_psll1BGeneric;
        vmp_psll1H =            vmp_psll1HGeneric;
        vmp_psll1H0 =           vmp_psll1HGeneric;
        vmp_psll1W =            vmp_psll1WGeneric;
        vmp_psll1W0 =           vmp_psll1WGeneric;

        vmp_psrlB =             vmp_psrlBGeneric;
        vmp_psrlB0 =            vmp_psrlBGeneric;
        vmp_psrlH =             vmp_psrlHGeneric;
        vmp_psrlH0 =            vmp_psrlHGeneric;
        vmp_psrlW =             vmp_psrlWGeneric;
        vmp_psrlW0 =            vmp_psrlWGeneric;

        vmp_psrl1B =            vmp_psrl1BGeneric;
        vmp_psrl1B0 =           vmp_psrl1BGeneric;
        vmp_psrl1H =            vmp_psrl1HGeneric;
        vmp_psrl1H0 =           vmp_psrl1HGeneric;
        vmp_psrl1W =            vmp_psrl1WGeneric;
        vmp_psrl1W0 =           vmp_psrl1WGeneric;

        vmp_prolB =             vmp_prolBGeneric;
        vmp_prolB0 =            vmp_prolBGeneric;
        vmp_prolH =             vmp_prolHGeneric;
        vmp_prolH0 =            vmp_prolHGeneric;
        vmp_prolW =             vmp_prolWGeneric;
        vmp_prolW0 =            vmp_prolWGeneric;

        vmp_prol1B =            vmp_prol1BGeneric;
        vmp_prol1B0 =           vmp_prol1BGeneric;
        vmp_prol1H =            vmp_prol1HGeneric;
        vmp_prol1H0 =           vmp_prol1HGeneric;
        vmp_prol1W =            vmp_prol1WGeneric;
        vmp_prol1W0 =           vmp_prol1WGeneric;
      }
}




