
// 
//  QVas3DX86.cpp - 3D (Quad) Vector Addition & Subtraction 
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "QVas3D.h"                                  // 3D Math
#include    "QVas3DEx.h"



//
//  Setup Vec3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void QVas3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);
    
    
    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_QVecAdd =           vmp_QVecAddAsmSSE;
        vmp_QVecAdd0 =          vmp_QVecAddAsmSSE0;
        vmp_QVecSub =           vmp_QVecSubAsmSSE;
        vmp_QVecSub0 =          vmp_QVecSubAsmSSE0;
        vmp_QVecAddScalar =     vmp_QVecAddScalarAsmSSE;
        vmp_QVecAddScalar0 =    vmp_QVecAddScalarAsmSSE0;
        vmp_QVecSubScalar =     vmp_QVecSubScalarAsmSSE;
        vmp_QVecSubScalar0 =    vmp_QVecSubScalarAsmSSE0;
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_QVecAdd =           vmp_QVecAddAsm3DNow;
        vmp_QVecAdd0 =          vmp_QVecAddAsm3DNow;
        vmp_QVecSub =           vmp_QVecSubAsm3DNow;
        vmp_QVecSub0 =          vmp_QVecSubAsm3DNow;
        vmp_QVecAddScalar =     vmp_QVecAddScalarAsm3DNow;
        vmp_QVecAddScalar0 =    vmp_QVecAddScalarAsm3DNow;
        vmp_QVecSubScalar =     vmp_QVecSubScalarAsm3DNow;
        vmp_QVecSubScalar0 =    vmp_QVecSubScalarAsm3DNow;
      }

    else
      {
        vmp_QVecAdd =           vmp_QVecAddGeneric;
        vmp_QVecAdd0 =          vmp_QVecAddGeneric;
        vmp_QVecSub =           vmp_QVecSubGeneric;
        vmp_QVecSub0 =          vmp_QVecSubGeneric;
        vmp_QVecAddScalar =     vmp_QVecAddScalarGeneric;
        vmp_QVecAddScalar0 =    vmp_QVecAddScalarGeneric;
        vmp_QVecSubScalar =     vmp_QVecSubScalarGeneric;
        vmp_QVecSubScalar0 =    vmp_QVecSubScalarGeneric;
      }
}




