
//
//  QVmd3D.cpp - (Quad) 3D Vector Multiplication & Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "QVmd3D.h"                                  // 3D Math
#include    "QVmd3DEx.h"



vmp_QVecMulProc             vmp_QVecMul;
vmp_QVecMulProc             vmp_QVecMul0;               // 128bit aligned
vmp_QVecMAddProc            vmp_QVecMAdd;
vmp_QVecMAddProc            vmp_QVecMAdd0;              // 128bit aligned
vmp_QVecMSubProc            vmp_QVecMSub;
vmp_QVecMSubProc            vmp_QVecMSub0;              // 128bit aligned
vmp_QVecScaleProc           vmp_QVecScale;
vmp_QVecScaleProc           vmp_QVecScale0;             // 128bit aligned
vmp_QVecDivProc             vmp_QVecDiv;
vmp_QVecDivProc             vmp_QVecDiv0;               // 128bit aligned
vmp_QVecDivProc             vmp_QVecDivFast;
vmp_QVecDivProc             vmp_QVecDivFast0;           // 128bit aligned
vmp_QDotProductProc         vmp_QDotProduct;
vmp_QDotProductProc         vmp_QDotProduct0;           // 128bit aligned
vmp_QCrossProductProc       vmp_QCrossProduct;
vmp_QCrossProductProc       vmp_QCrossProduct0;         // 128bit aligned





//
//  (Quad) Vector Product (Multiplication)
//

void vmp_QVecMulGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    pvD->x = pvA->x * pvB->x;
    pvD->y = pvA->y * pvB->y;
    pvD->z = pvA->z * pvB->z;
    pvD->w = pvA->w * pvB->w;
}



//
//  (Quad) Vector Product Sum (Multiplication-Add)
//

void vmp_QVecMAddGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA,
                   const vmp3DQVector * const pvB, const vmp3DQVector * const pvC)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    ASSERT_PTR4(pvC);

    pvD->x = pvA->x * pvB->x + pvC->x;
    pvD->y = pvA->y * pvB->y + pvC->y;
    pvD->z = pvA->z * pvB->z + pvC->z;
    pvD->w = pvA->w * pvB->w + pvC->w;
}



//
//  (Quad) Vector Product Sub (Multiplication-Sub)
//

void vmp_QVecMSubGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA,
                   const vmp3DQVector * const pvB, const vmp3DQVector * const pvC)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    ASSERT_PTR4(pvC);

    pvD->x = pvA->x * pvB->x - pvC->x;
    pvD->y = pvA->y * pvB->y - pvC->y;
    pvD->z = pvA->z * pvB->z - pvC->z;
    pvD->w = pvA->w * pvB->w - pvC->w;
}



//
//  (Quad) Scale Vector w/Scalar
//

void vmp_QVecScaleGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, float fScalar )
{
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);

    pvD->x = pvA->x * fScalar;
    pvD->y = pvA->y * fScalar;
    pvD->z = pvA->z * fScalar;
    pvD->w = pvA->w * fScalar;
}



//
//  (Quad) Vector (Division)
//

void vmp_QVecDivGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, float fScalar)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_FZERO(fScalar);

    pvD->x = pvA->x / fScalar;
    pvD->y = pvA->y / fScalar;
    pvD->z = pvA->z / fScalar;
    pvD->w = pvA->w / fScalar;
}



//
//  Dot Product
//

void vmp_QDotProductGeneric(float * const pfD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{
    ASSERT_PTR4(pfD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    *pfD = pvA->x * pvB->x + pvA->y * pvB->y + pvA->z * pvB->z;
}



//
//  (Quad) Cross Product
//

void vmp_QCrossProductGeneric(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{   
    float x, y;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    x = pvA->y * pvB->z - pvA->z * pvB->y;
    y = pvA->z * pvB->x - pvA->x * pvB->z;
    pvD->z = pvA->x * pvB->y - pvA->y * pvB->x;
    pvD->x = x; 
    pvD->y = y;
    pvD->w = 0.0f;
}





