
//
//  Vmd3D.cpp - 3D Vector Multiplication & Division
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "vmp3D.h"
#include    "vmp3DEx.h"
#include    "Vmd3D.h"                                   // 3D Math
#include    "Vmd3DEx.h"



vmp_VecMulProc              vmp_VecMul;
vmp_VecMulProc              vmp_VecMul0;                // 128bit aligned
vmp_VecMAddProc             vmp_VecMAdd;
vmp_VecMAddProc             vmp_VecMAdd0;               // 128bit aligned
vmp_VecMSubProc             vmp_VecMSub;
vmp_VecMSubProc             vmp_VecMSub0;               // 128bit aligned
vmp_VecDivProc              vmp_VecDiv;
vmp_VecDivProc              vmp_VecDiv0;                // 128bit aligned
vmp_VecDivProc              vmp_VecDivFast;
vmp_VecDivProc              vmp_VecDivFast0;            // 128bit aligned
vmp_VecScaleProc            vmp_VecScale;
vmp_VecScaleProc            vmp_VecScale0;              // 128bit aligned
vmp_DotProductProc          vmp_DotProduct;
vmp_DotProductProc          vmp_DotProduct0;            // 128bit aligned
vmp_CrossProductProc        vmp_CrossProduct;
vmp_CrossProductProc        vmp_CrossProduct0;          // 128bit aligned





//
//  Vector Product (Multiplication)
//

void vmp_VecMulGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    pvD->x = pvA->x * pvB->x;
    pvD->y = pvA->y * pvB->y;
    pvD->z = pvA->z * pvB->z;
}



//
//  Vector Product Sum (Multiplication-Add)
//

void vmp_VecMAddGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA,
                  const vmp3DVector * const pvB, const vmp3DVector * const pvC)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    ASSERT_PTR4(pvC);

    pvD->x = pvA->x * pvB->x + pvC->x;
    pvD->y = pvA->y * pvB->y + pvC->y;
    pvD->z = pvA->z * pvB->z + pvC->z;
}



//
//  Vector Product Sub (Multiplication-Sub)
//

void vmp_VecMSubGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA,
                  const vmp3DVector * const pvB, const vmp3DVector * const pvC)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    ASSERT_PTR4(pvC);

    pvD->x = pvA->x * pvB->x - pvC->x;
    pvD->y = pvA->y * pvB->y - pvC->y;
    pvD->z = pvA->z * pvB->z - pvC->z;
}



//
//  Scale Vector w/Scalar
//

void vmp_VecScaleGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA, float fScalar )
{
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);

    pvD->x = pvA->x * fScalar;
    pvD->y = pvA->y * fScalar;
    pvD->z = pvA->z * fScalar;
}



//
//  Vector Scalar Division
//

void vmp_VecDivGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA, float fScalar )
{
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_FZERO(fScalar);

    pvD->x = pvA->x / fScalar;
    pvD->y = pvA->y / fScalar;
    pvD->z = pvA->z / fScalar;
}


//
//  Dot Product
//

void vmp_DotProductGeneric(float * const pfD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
{
    ASSERT_PTR4(pfD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    *pfD = pvA->x * pvB->x + pvA->y * pvB->y + pvA->z * pvB->z;
}



//
//  Cross Product
//

void vmp_CrossProductGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA, const vmp3DVector * const pvB)
{   
    float x, y;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    x = pvA->y * pvB->z - pvA->z * pvB->y;
    y = pvA->z * pvB->x - pvA->x * pvB->z;
    pvD->z = pvA->x * pvB->y - pvA->y * pvB->x;
    pvD->x = x; 
    pvD->y = y;
}






