        TITLE   FsfX86.asm - 80(x)86 3D Math Special Functions
        PAGE    53,160

;  Fsf.asm - Math Multiplication & Division
;
;  This document contains file that you may use at your own risk in your own programs.
;
;  This module is designed to accomodate the book.
;
;               This module is designed for fast 3D math
;
;  Created - 3 Oct 2001 - James Leiterman
;
;  Last Modification
;
;       Tabs = 8



        .XLIST

        .686
        .XMM

        .model flat, C

;ifdef _DEBUG
USE_ASSERT      =       1
;endif


        include KariType.inc
        include vmp3DX86M.inc
 
        .LIST


        .data
        align   16

ONES    REAL4   1.0,1.0,1.0,1.0
vTiny   REAL4   0.0000001,0.0000001,0.0000001,0.0000001
vZero   REAL4   0.0,0.0,0.0,0.0


;------------------------------------------------------------------------------------------------


FsfX86  SEGMENT USE32 PUBLIC 'CODE'

;-------------------------------------------------------------------------------------------------




;_________________________________________________________________________________________________
;=================================================================================================
;       (float) Square Root
;
;void vmp_FSqrt( float * const pfD, float fA )
;{
;       assert(NULL!=pfD);
;
;       *pfD = sqrtf( fA );
;}

;_________________________________________________________________________________________________

; *** 3DNow version ***  (FAST) 15bit precision
;
        align   16
        public  vmp_FSqrtFastAsm3DNow
vmp_FSqrtFastAsm3DNow       proc    near , pfD:ptr, fA:REAL4
        .K3D
;;;        femms

ASSERT_PTR(pfD)
ASSERT_FNEG(fA)

        mov     eax,pfD                         ; Vector Destination
        movd    mm0,fA                          ;       {0    fA}

         ;      pfrsqrt=15bit accuracy
        pfrsqrt  mm1,mm0                 ;X0=f(x)=1/sqrt(x)  {1/sqrt(fA)  1/sqrt(fA)}
        pfmul    mm0,mm1                ;             {0/sqrt(fA) A/sqrt(fA)}
                                ;             {    0       sqrt(fA) }   
        movd    [eax],mm0       ;                     sqrt fA
;;;        femms
        ret
vmp_FSqrtFastAsm3DNow endp

;_________________________________________________________________________________________________

; *** 3DNow version ***   24bit precision
;
        align   16
        public  vmp_FSqrtAsm3DNow
vmp_FSqrtAsm3DNow       proc    near , pfD:ptr, fA:REAL4
        .K3D
;;;        femms

ASSERT_PTR(pfD)
ASSERT_FNEG(fA)

        movd    mm0,fA                          ;       {0    fA}
        mov     eax,pfD                         ; Vector Destination

         ;      pfrsqrt=15bit accuracy
        pfrsqrt  mm1,mm0                 ;X0=f(x)=1/sqrt(x)  {1/sqrt(fA)  1/sqrt(fA)}

         ;      refine Newton-Raphson reciprocal square approximation
         ;      Calculate 1/sqrt() accurate to 24bits
        movq     mm2,mm1                ;            {1/sqrt(fA)  1/sqrt(fA)}
        pfmul    mm1,mm1                ;X1=f(x0,x0) {1/fA        1/fA}
        pfrsqit1 mm1,mm0                ;X2=f(x,x1)     {1st step}

         ;      Calculate sqrt() = 1/(1/sqrt())  24bit accuracy
        pfrcpit2 mm1,mm2                ; y=f(x2,x0)  1/(sqrt #)
        pfmul    mm0,mm1                ;             {0/sqrt(fA) A/sqrt(fA)}
                                ;             {    0       sqrt(fA) }   
        movd    [eax],mm0       ;                     sqrt fA
;;;        femms
        ret
vmp_FSqrtAsm3DNow endp

;_________________________________________________________________________________________________

; *** SSE version ***

        align   16
        public  vmp_FSqrtAsmSSE
vmp_FSqrtAsmSSE       proc    near , pfD:ptr, fA:REAL4
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_FNEG(fA)

        movss   xmm0,fA                         ;        {# # # A}
        mov     eax,pfD                         ; (float) Destination
        sqrtss  xmm0,xmm0                       ; SqRoot {# # # sqrt(fA)}

        movss   [eax],xmm0                      ; Save Square Root

;;;        emms
        ret
vmp_FSqrtAsmSSE endp


;_________________________________________________________________________________________________

; *** SSE version ***

        align   16
        public  vmp_FSqrtFastAsmSSE
vmp_FSqrtFastAsmSSE       proc    near , pfD:ptr, fA:REAL4
        .XMM
;;;        emms
ASSERT_PTR(pfD)
ASSERT_FNEG(fA)

        movss   xmm0,fA                         ;        {# # # A}
        mov     eax,pfD                         ; (float) Destination

        movaps  xmm1,xmm0
        movaps  xmm2,xmm0
        movaps  xmm3,xmm0

        cmpss   xmm2,vTiny,5    ; >=  1's = #'s Okay
        cmpss   xmm3,vTiny,1    ; <   1's = too close to zero

        rsqrtss xmm0,xmm0                       ;        {0 0 0 1/s}

        ;       Correct for infinity due to 1/0 =  1/sqrt(0)

        andps   xmm3,ONES       ; Preserve 1.0's (in positions that were infinite!)
        andps   xmm0,xmm2       ; Preserve #'s that are not too small, else zeros
        orps    xmm0,xmm3       ; Blended #'s and 1.0's

        mulss   xmm0,xmm1       ; {  ... Ax*1/sqrt(Ax)}

        movss   [eax],xmm0                      ; Save Square Root

;;;        emms
        ret
vmp_FSqrtFastAsmSSE endp




FsfX86  ends
        end

