
//
//  Bench.cpp - Vector 3D Special Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include <iostream.h>

    //  Book Development Library Modules
    
#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"

#include "Vsf3D.h"
#include "Vsf3DEx.h"



typedef enum
{
    iTEST_MAGNITUDE,
    iTEST_MAGNITUDE_FAST,
    iTEST_NORMALIZE,
    iTEST_NORMALIZE_FAST,
    iTEST_VEC_SQRT,
    iTEST_VEC_SQRT_FAST,
    iTEST_MIN,
    iTEST_MAX
} iTEST;




//
//  Test 
//

static void testVec( bool bAlign, iTEST iFunc )
{
    vmp3DVector *pvA, *pvB, *pvTmp, *pvTmp2;
    int x, y, z, dif;
    float f, f2, fPrecision, fScalar, fWTst;
    float buf[4 *4  +3];            // enough space + 12 bytes (3 floats) extra
    bool bF;

        // Force proper alignment (Crude just for this demo)

    bF = false;

    if ( bAlign )
      {
        pvA    = (vmp3DVector*) ALIGN16((int)(buf));
        pvB    = (vmp3DVector*) ALIGN16((int)(pvA+1));
        pvTmp  = (vmp3DVector*) ALIGN16((int)(pvB+1));
        pvTmp2 = (vmp3DVector*) ALIGN16((int)(pvTmp+1));
      }
    else
      {
        pvA = (vmp3DVector*) buf;
        if (!(0x000f & (int)pvA))
          {
            pvA = (vmp3DVector*) (buf+1);           // Make sure misaligned!
          }

        pvB    = pvA + 1;
        pvTmp2 = pvB + 1;
        pvTmp  = pvTmp2 + 1;
      }

    dif = 4;
    fPrecision = SINGLE_PRECISION;
    
    for ( z = -dif; z <= dif; z++ )
      {
        pvA->z = 0.123f *(float)z;
        pvB->y = 0.123f *(float)z;

        for ( y = -dif; y <= dif; y++ )
          {
            pvA->y = 0.123f *(float)y;
            pvB->x = 0.123f *(float)y;

            for ( x = -dif; x <= dif; x++ )
              { 
                pvA->x = 0.123f *(float)x;
                pvB->z = 0.123f *(float)y;
                fScalar = pvA->x;

                fWTst = ((vmp3DQVector *) pvTmp)->w = fScalar;  // Get untouchable value!

                switch(iFunc)
                  {
                    case iTEST_VEC_SQRT:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        if ( pvA->x < 0.0f ) pvA->x = -pvA->x;
                        if ( pvA->y < 0.0f ) pvA->y = -pvA->y;
                        if ( pvA->z < 0.0f ) pvA->z = -pvA->z;

                        vmp_VecSqrtGeneric( pvTmp2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecSqrt( pvTmp, pvA );      // Aligned
                        else
                            vmp_VecSqrt0( pvTmp, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_VEC_SQRT_FAST:
                        fPrecision = FAST_PRECISION;
                        bF = false;
                        if ( pvA->x < 0.0f ) pvA->x = -pvA->x;
                        if ( pvA->y < 0.0f ) pvA->y = -pvA->y;
                        if ( pvA->z < 0.0f ) pvA->z = -pvA->z;

                        vmp_VecSqrtGeneric( pvTmp2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecSqrtFast( pvTmp, pvA );      // Aligned
                        else
                            vmp_VecSqrtFast0( pvTmp, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_MAGNITUDE:
                        fPrecision = SINGLE_PRECISION;
                        bF = true;
                        vmp_VecMagnitudeGeneric( &f2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMagnitude( &f, pvA );        // Aligned
                        else
                            vmp_VecMagnitude0( &f, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_MAGNITUDE_FAST:
                        fPrecision = FAST_PRECISION;
                        bF = true;
                        vmp_VecMagnitudeGeneric( &f2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMagnitudeFast( &f, pvA );        // Aligned
                        else
                            vmp_VecMagnitudeFast0( &f, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_NORMALIZE:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecNormalizeGeneric( pvTmp2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecNormalize( pvTmp, pvA ); // Aligned
                        else
                            vmp_VecNormalize0( pvTmp, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_NORMALIZE_FAST:
                        fPrecision = FAST_PRECISION;
                        bF = false;
                        vmp_VecNormalizeGeneric( pvTmp2, pvA );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecNormalizeFast( pvTmp, pvA ); // Aligned
                        else
                            vmp_VecNormalizeFast0( pvTmp, pvA );
                        vmp_SIMDExit();
                        break;

                    case iTEST_MIN:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecMinGeneric( pvTmp2, pvA, pvB );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMin( pvTmp, pvA, pvB );  // Aligned
                        else
                            vmp_VecMin0( pvTmp, pvA, pvB );
                        vmp_SIMDExit();
                        break;

                    case iTEST_MAX:
                        fPrecision = SINGLE_PRECISION;
                        bF = false;
                        vmp_VecMaxGeneric( pvTmp2, pvA, pvB );
                        vmp_SIMDEntry();
                        if (bAlign)
                            vmp_VecMax( pvTmp, pvA, pvB );  // Aligned
                        else
                            vmp_VecMax0( pvTmp, pvA, pvB );
                        vmp_SIMDExit();
                        break;
                  }
                if (bF)
                  {
                    if (!vmp_IsFEqual(f, f2, fPrecision))
                      {
                        cout << " vA.x:" << pvA->x << " vA.y:" << pvA->y << " vA.z:" << pvA->z << "\n"; 
                        cout << " vB.x:" << pvB->x << " vB.y:" << pvB->y << " vB.z:" << pvB->z << "\n"; 
                        cout << " " << f2 << " Good vs Bad! " << f << "  *** MISMATCH!!! ***\n" << endl;
                      }
                  }
                else
                  {
                    if (!vmp_IsVEqual(pvTmp, pvTmp2, fPrecision)) 
                      {
                        cout << " vA.x:" << pvA->x << " vA.y:" << pvA->y << " vA.z:" << pvA->z << "\n";
                        cout << " vB.x:" << pvB->x << " vB.y:" << pvB->y << " vB.z:" << pvB->z << "\n"; 
                        cout << "    x:" << pvTmp2->x << " y:" << pvTmp2->y << " z:" << pvTmp2->z << "  Good vs Bad!!!\n";
                        cout << "    x:" << pvTmp->x << " y:" << pvTmp->y << " z:" << pvTmp->z << "   *** MISMATCH!!! ***\n" << endl;
                      }

                        //  Check for '.W' contamination
                    if (!vmp_IsFEqual(fWTst, ((vmp3DQVector *) pvTmp)->w, fPrecision ))
                      {
                        cout << ".W is contaminated! should be: " << fWTst << " not " << ((vmp3DQVector *) pvTmp)->w << endl;
                      }
                  }
              }
          }
      }
}



int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU

    cout << "\nVector Special Function Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    Vsf3DSetup( &cinfo );                               // Setup our 3D Vector stuff

    cout << "Testing VecSqrt" << endl;
    testVec( false, iTEST_VEC_SQRT );

    cout << "Testing VecSqrt" << endl;
    testVec( true,  iTEST_VEC_SQRT );

    cout << "Testing VecSqrt (FAST)" << endl;
    testVec( false, iTEST_VEC_SQRT_FAST );

    cout << "Testing VecSqrt (FAST)" << endl;
    testVec( true,  iTEST_VEC_SQRT_FAST );

    cout << "Testing VecMin" << endl;
    testVec( false, iTEST_MIN );

    cout << "Testing VecMin (Aligned)" << endl;
    testVec( true,  iTEST_MIN );

    cout << "Testing VecMax" << endl;
    testVec( false, iTEST_MAX );

    cout << "Testing VecMax (Aligned)" << endl;
    testVec( true,  iTEST_MAX );

    cout << "Testing VecMagnitude" << endl;
    testVec( false, iTEST_MAGNITUDE );

    cout << "Testing VecMagnitude (Aligned)" << endl;
    testVec( true,  iTEST_MAGNITUDE );

    cout << "Testing VecMagnitude (FAST) possible Low Precision" << endl;
    testVec( false, iTEST_MAGNITUDE_FAST );

    cout << "Testing VecMagnitude (Aligned) (FAST) possible Low Precision" << endl;
    testVec( true,  iTEST_MAGNITUDE_FAST );

    cout << "Testing VecNormalize" << endl;
    testVec( false, iTEST_NORMALIZE );

    cout << "Testing VecNormalize (Aligned)" << endl;
    testVec( true,  iTEST_NORMALIZE );

    cout << "Testing VecNormalize (FAST) possible Low Precision" << endl;
    testVec( false, iTEST_NORMALIZE_FAST );

    cout << "Testing VecNormalize (Aligned) (FAST) possible Low Precision" << endl;
    testVec( true,  iTEST_NORMALIZE_FAST );

    cout << "Fin!\n\n" << flush;

    return 0;
}

