
//
//  Vsf3DX86.cpp - 3D Vector Special Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  NOTE: Because these a 3 float Vectors the 4th float has to be handled as 
//      a special case thus slowing the code down. This is the same as an AoS 
//      orientation. Instead a SoA should be used in conjunction with the quad
//      set of functions.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <math.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"
#include    "vmp3DEx.h"

#include    "Vsf3D.h"                                   // 3D Math
#include    "Vsf3DEx.h"


//
//  Setup Vec3D module
//
//  NOTE: This function sets up the function vectors so the best function
//      is attached to the vector for the processor running.
//

void Vsf3DSetup( const CpuInfo * const pcinfo )
{
    ASSERT_PTR4(pcinfo);

    if ( CPUBITS_SSE & pcinfo->nBits )
      {
        vmp_VecSqrt =           vmp_VecSqrtAsmSSE;
        vmp_VecSqrt0 =          vmp_VecSqrtAsmSSE0;
        vmp_VecSqrtFast =       vmp_VecSqrtFastAsmSSE;          // ***FAST***
        vmp_VecSqrtFast0 =      vmp_VecSqrtFastAsmSSE0;         // ***FAST***

        vmp_VecMin =            vmp_VecMinAsmSSE;
        vmp_VecMin0 =           vmp_VecMinAsmSSE0;
        vmp_VecMax =            vmp_VecMaxAsmSSE;
        vmp_VecMax0 =           vmp_VecMaxAsmSSE0;

        vmp_VecMagnitude =      vmp_VecMagnitudeAsmSSE;
        vmp_VecMagnitude0 =     vmp_VecMagnitudeAsmSSE0;
        vmp_VecMagnitudeFast =  vmp_VecMagnitudeFastAsmSSE;     // ***FAST***
        vmp_VecMagnitudeFast0 = vmp_VecMagnitudeFastAsmSSE0;    // ***FAST***

        vmp_VecNormalize =      vmp_VecNormalizeAsmSSE;
        vmp_VecNormalize0 =     vmp_VecNormalizeAsmSSE0;
        vmp_VecNormalizeFast =  vmp_VecNormalizeFastAsmSSE;     // ***FAST***
        vmp_VecNormalizeFast0 = vmp_VecNormalizeFastAsmSSE0;    // ***FAST***
      }

    else if ( CPUBITS_3DNOW & pcinfo->nBits )
      {
        vmp_VecSqrt =           vmp_VecSqrtAsm3DNow;
        vmp_VecSqrt0 =          vmp_VecSqrtAsm3DNow;
        vmp_VecSqrtFast =       vmp_VecSqrtFastAsm3DNow;        // ***FAST***
        vmp_VecSqrtFast0 =      vmp_VecSqrtFastAsm3DNow;        // ***FAST***

        vmp_VecMin =            vmp_VecMinAsm3DNow;
        vmp_VecMin0 =           vmp_VecMinAsm3DNow;
        vmp_VecMax =            vmp_VecMaxAsm3DNow;
        vmp_VecMax0 =           vmp_VecMaxAsm3DNow;

        vmp_VecMagnitude =      vmp_VecMagnitudeAsm3DNow;
        vmp_VecMagnitude0 =     vmp_VecMagnitudeAsm3DNow;
        vmp_VecMagnitudeFast =  vmp_VecMagnitudeFastAsm3DNow;   // ***FAST***
        vmp_VecMagnitudeFast0 = vmp_VecMagnitudeFastAsm3DNow;   // ***FAST***

        vmp_VecNormalize =      vmp_VecNormalizeAsm3DNow;
        vmp_VecNormalize0 =     vmp_VecNormalizeAsm3DNow;
        vmp_VecNormalizeFast =  vmp_VecNormalizeFastAsm3DNow;   // ***FAST***
        vmp_VecNormalizeFast0 = vmp_VecNormalizeFastAsm3DNow;   // ***FAST***
      }
    
    else
      {
        vmp_VecSqrt =           vmp_VecSqrtGeneric;
        vmp_VecSqrt0 =          vmp_VecSqrtGeneric;
        vmp_VecSqrtFast =       vmp_VecSqrtGeneric;
        vmp_VecSqrtFast0 =      vmp_VecSqrtGeneric;

        vmp_VecMin =            vmp_VecMinGeneric;
        vmp_VecMin0 =           vmp_VecMinGeneric;
        vmp_VecMax =            vmp_VecMaxGeneric;
        vmp_VecMax0 =           vmp_VecMaxGeneric;

        vmp_VecMagnitude =      vmp_VecMagnitudeGeneric;
        vmp_VecMagnitude0 =     vmp_VecMagnitudeGeneric;
        vmp_VecMagnitudeFast =  vmp_VecMagnitudeGeneric;
        vmp_VecMagnitudeFast0 = vmp_VecMagnitudeGeneric;

        vmp_VecNormalize =      vmp_VecNormalizeGeneric;
        vmp_VecNormalize0 =     vmp_VecNormalizeGeneric;
        vmp_VecNormalizeFast =  vmp_VecNormalizeGeneric;
        vmp_VecNormalizeFast0 = vmp_VecNormalizeGeneric;
      }
}



