
//
//  Bench.cpp - House Vertices
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//      Hopefully by now you have realized the necessity of 16 byte aligned data
//      and so this program is written as such!
//
//      Because of all the Generic code there is an excessive amount of SIMD entry/exit
//      function calls. This gets dramatically reduced by
//
//  Uses: Chap03\cpuid
//        Chap07\qVas3d
//        Chap11\matrix3d
//        Chap15\vecBug
//
//  Created - 5 Oct 2001 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//

#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>
#include    <memory.h>
#include <D3dx8math.h>
#include <stdio.h>

    //  Book Development Library Modules

#include "CpuAsm.h"                                     // CPU Module
#include "vmp3D.h"
#include "vecBug.h"



typedef enum
{
    iTEST_XZY = 0,
    iTEST_XYZ_LEFT,
} iTEST;



//  ASE - XZY Vertex List
//
//      *MESH_VERTEX_LIST {
//          *MESH_VERTEX    0   -3.5430 -2.4530 -0.0000

vmp3DVector HouseVertexList[] = 
{   //     X        Z         Y
    { -3.5430f, -2.4530f, -0.0000f },   // Vertex #0
    {  3.5382f, -2.4530f, -0.0000f },
    { -3.5430f,  2.4445f,  0.0000f },
    {  3.5382f,  2.4445f,  0.0000f },
    { -3.5430f, -0.0051f,  4.2178f },
    {  3.5382f, -0.0051f,  4.2178f },
    { -3.5430f, -2.4530f,  2.4776f },
    {  3.5382f, -2.4530f,  2.4776f },
    {  3.5382f,  2.4445f,  2.4776f },
    { -3.5430f,  2.4445f,  2.4776f }    // Vertex #9
};

#define HOUSE_VERTEXLIST_MAX (sizeof(HouseVertexList) / sizeof(vmp3DVector))    // 10



//  Face Vertex Indices
//
//      *MESH_FACE_LIST {
//          *MESH_FACE    0:    A:    0 B:    2 C:    3 AB:    1 BC:    1 CA:    0   *MESH_SMOOTHING 1  *MESH_MTLID 0

uint HouseFaceList[] = 
{ // A  B  C
     0, 2, 3,               // Face #0 
     3, 1, 0, 
     0, 1, 7, 
     7, 6, 0, 
     6, 7, 5, 
     5, 4, 6, 
     1, 3, 8, 
     8, 7, 1, 
     7, 8, 5, 
     3, 2, 9, 
     9, 8, 3, 
     8, 9, 4, 
     4, 5, 8, 
     2, 0, 6, 
     6, 9, 2, 
     9, 6, 4                // Face #15
};

#define HOUSE_FACELIST_MAX  (sizeof(HouseFaceList) / (sizeof(uint)*3))  // 16



//  UV Texture coordinates
//
//      *MESH_TVERTLIST {
//          *MESH_TVERT 0   0.0005  0.7518  0.0005

vmp3DVector HouseTVertList[] =
{   //   U         V       W
    { 0.0005f,  0.7518f,    0.0005f },  // 0
    { 0.7456f,  0.7518f,    0.0005f },  // 1
    { 0.7456f,  1.0015f,    0.9995f },  // 2
    { 0.0005f,  1.0015f,    0.9995f },  // 3
    { 0.0044f,  0.5058f,    0.0005f },  // 4
    { 0.7450f,  0.5058f,    0.0005f },  // 5
    { 0.7450f,  0.7481f,    0.9995f },  // 6
    { 0.0044f,  0.7481f,    0.9995f },  // 7
    { 0.2545f,  0.5065f,    0.9995f },  // 8
    { 0.7455f,  0.5065f,    0.9995f },  // 9
    { 0.7455f,  0.7949f,    0.9995f },  // 10
    { 0.2545f,  0.7949f,    0.9995f },  // 11
    { 0.5001f,  0.9975f,    0.9995f },  // 12
    { 0.2515f,  0.5039f,    0.0005f },  // 13
    { 0.7486f,  0.5039f,    0.0005f },  // 14
    { 0.7486f,  0.7958f,    0.0005f },  // 15
    { 0.2515f,  0.7958f,    0.0005f },  // 16
    { 0.5000f,  1.0009f,    0.0005f },  // 17
    { 0.0005f,  0.5057f,    0.0000f },  // 18
    { 0.9995f,  0.5057f,    1.0000f },  // 19
    { 0.9995f,  0.7455f,    1.0000f },  // 20
    { 0.0005f,  0.7455f,    0.0000f },  // 21
    { 0.0005f,  0.7508f,    0.5000f },  // 22
    { 0.9995f,  0.7508f,    0.2500f },  // 23
    { 0.9995f,  0.9984f,    0.2500f },  // 24
    { 0.0005f,  0.9984f,    0.7500f },  // 25
    { 0.5034f,  0.0047f,    0.0005f },  // 26
    { 0.5034f,  0.2465f,    0.9995f },  // 27
    { 0.7452f,  0.2465f,    0.9995f },  // 28
    { 0.7452f,  0.0047f,    0.0005f }   // 29
};

#define HOUSE_TVERTLIST_MAX (sizeof(HouseTVertList) / sizeof(vmp3DVector))  // 30                                   // 30



//
//  Face to Texture Vertex Index
//
//      *MESH_TFACELIST {
//          *MESH_TFACE 0   56  57  58

uint HouseTFaceList[] = 
{ // A   B   C  (TVert Index)
    26, 27, 28,         // Face #0
    28, 29, 26,         // 1
    22, 23, 24,         // 2
    24, 25, 22,         // 3
     4,  5,  6,         // 4
     6,  7,  4,         // 5
    13, 14, 15,         // 6
    15, 16, 13,         // 7
    16, 15, 17,         // 8
    18, 19, 20,         // 9
    20, 21, 18,         // 10
     0,  1,  2,         // 11
     2,  3,  0,         // 12
     8,  9, 10,         // 13
    10, 11,  8,         // 14
    11, 10, 12          // Face #15
};

#define HOUSE_NUMTFACELIST_MAX  (sizeof(HouseTFaceList) / (sizeof(uint)*3)) // 16



//  XZY Normals
//
//      *MESH_NORMALS {
//          *MESH_FACENORMAL 0  0.0000  0.0000  -1.0000

vmp3DVector HouseNormals[] = 
{   //   X         Z       Y
    {  0.0000f,  0.0000f, -1.0000f },   // #0
    {  0.0000f,  0.0000f, -1.0000f },   // #1
    {  0.0000f, -1.0000f,  0.0000f },   // #2
    {  0.0000f, -1.0000f, -0.0000f },
    {  0.0000f, -0.5794f,  0.8150f },
    {  0.0000f, -0.5794f,  0.8150f },
    {  1.0000f,  0.0000f,  0.0000f },
    {  1.0000f,  0.0000f,  0.0000f },
    {  1.0000f,  0.0000f,  0.0000f },
    { -0.0000f,  1.0000f,  0.0000f },
    { -0.0000f,  1.0000f,  0.0000f },
    { -0.0000f,  0.5792f,  0.8152f },
    {  0.0000f,  0.5792f,  0.8152f },
    { -1.0000f, -0.0000f,  0.0000f },
    { -1.0000f, -0.0000f,  0.0000f },
    { -1.0000f, -0.0000f, -0.0000f }    // #15
};

#define HOUSE_NORMALS_MAX   (sizeof(HouseNormals) / sizeof(vmp3DVector))    // 10



void vmp_VecNormalizeGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA);




//
//  (Quad) Vector Difference (Subtraction)
//

void vmp_QVecSub(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{   
    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    pvD->x = pvA->x - pvB->x;
    pvD->y = pvA->y - pvB->y;
    pvD->z = pvA->z - pvB->z;
    pvD->w = pvA->w - pvB->w;
}



//
//  Dot Product
//

void vmp_QDotProduct(float * const pfD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{
    ASSERT_PTR4(pfD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    *pfD = pvA->x * pvB->x + pvA->y * pvB->y + pvA->z * pvB->z;
}



//
//  (Quad) Cross Product
//

void vmp_QCrossProduct(vmp3DQVector * const pvD, const vmp3DQVector * const pvA, const vmp3DQVector * const pvB)
{   
    float x, y;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);

    x = pvA->y * pvB->z - pvA->z * pvB->y;
    y = pvA->z * pvB->x - pvA->x * pvB->z;
    pvD->z = pvA->x * pvB->y - pvA->y * pvB->x;
    pvD->x = x; 
    pvD->y = y;
    pvD->w = 0.0f;
}



//
//  Vector Normalization
//

void vmp_VecNormalizeGeneric(vmp3DVector * const pvD, const vmp3DVector * const pvA)
{
    float fMag;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);

    fMag = pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z; // + pvA->w * pvA->w;

    if ( fMag < SINGLE_PRECISION )  // If too close to zero...
      {
        pvD->x = pvA->x;
        pvD->y = pvA->y;
        pvD->z = pvA->z;
      }
    else
      {
        fMag = 1.0f / sqrtf(fMag);
        pvD->x = pvA->x * fMag;
        pvD->y = pvA->y * fMag;
        pvD->z = pvA->z * fMag;
      }
}



//
//  Vector Normalization
//

void vmp_QVecNormalize(vmp3DQVector * const pvD, const vmp3DQVector * const pvA)
{
    float fMag;

    ASSERT_PTR4(pvD);
    ASSERT_PTR4(pvA);

    fMag = pvA->x * pvA->x + pvA->y * pvA->y + pvA->z * pvA->z + pvA->w * pvA->w;

    if ( fMag < SINGLE_PRECISION )  // If too close to zero...
      {
        pvD->x = pvA->x;
        pvD->y = pvA->y;
        pvD->z = pvA->z;
      }
    else
      {
        fMag = 1.0f / sqrtf(fMag);
        pvD->x = pvA->x * fMag;
        pvD->y = pvA->y * fMag;
        pvD->z = pvA->z * fMag;
      }

    pvD->w = 0.0f;
}



static void testHouse( iTEST iTest )
{
    uint n, *ivptr, *itptr;
    float matrixbuf[ (HOUSE_TVERTLIST_MAX+HOUSE_VERTEXLIST_MAX+3+3+3+3+3)     *4  +3];  // enough space + 128bit alignment padding
    vmp3DQVector *pQHouse, *pQTex;
    vmp3DQVector *pQVec, *pNorm, *pvAB, *pvBC, *pvCA, *pvNAB, *pvNBC, *pvNCA;
    vmp3DQVector *pvBA, *pvCB, *pvAC;
    vmp3DQVector *pvXABC, *pvXBCA, *pvXCAB;
    vmp3DQVector *pvXNABC, *pvXNBCA, *pvXNCAB;
    vmp3DVector *pVec;
    char *hitA, *hitB;

        //  Align Data

    pQHouse  = (vmp3DQVector*) ALIGN16((int)matrixbuf);
    pQTex    = pQHouse + HOUSE_VERTEXLIST_MAX;
    pNorm    = pQTex   + HOUSE_TVERTLIST_MAX;
    pvAB = pNorm+1;
    pvBC = pvAB + 1;
    pvCA = pvBC + 1;

    pvBA = pvCA + 1;
    pvCB = pvBA + 1;
    pvAC = pvCB + 1;

    pvNAB = pvAC + 1;
    pvNBC = pvNAB + 1;
    pvNCA = pvNBC + 1;

    pvXABC = pvNCA + 1;
    pvXBCA = pvXABC + 1;
    pvXCAB = pvXBCA + 1;

    pvXNABC = pvXCAB + 1;
    pvXNBCA = pvXNABC + 1;
    pvXNCAB = pvXNBCA + 1;

    hitA = hitB = "";

    switch (iTest)
      {
        case iTEST_XZY:
            hitA = "**********";
            break;

        case iTEST_XYZ_LEFT:
            hitB = "**********";
            break;
      }

        //  Copy (and swizzle) House Vertex list

    pVec = HouseVertexList; 
    pQVec = pQHouse;

    for (n = 0; n < HOUSE_VERTEXLIST_MAX; n++)
      {
        if (iTEST_XZY == iTest)     // XZY ASE orientation
          {
            pQVec->y = pVec->y;
            pQVec->z = pVec->z;
          }
        else
          {
            pQVec->z = pVec->y;
            pQVec->y = pVec->z;
          }

        pQVec->x = pVec->x;
        pQVec->w = 0.0f;

        printf( "V[%u]: %f %f %f\n", n, pQVec->x, pQVec->y, pQVec->z );

        pVec++;
        pQVec++;
      }

    printf( "\n" );

        //  Copy (and swizzle) House Texture Vertice UV information

    pVec = HouseTVertList;  
    pQVec = pQTex;

    for (n = 0; n < HOUSE_TVERTLIST_MAX; n++)
      {
        if ( iTEST_XZY == iTest)    // XZY ASE orientation
          {
            pQVec->y = pVec->y;
            pQVec->z = pVec->z;
          }
        else
          {
            pQVec->z = pVec->y;
            pQVec->y = pVec->z;
          }

        pQVec->x = pVec->x;
        pQVec->w = 0.0f;

        printf( "T[%u]: %f %f %f\n", n, pQVec->x, pQVec->y, pQVec->z );

        pVec++;
        pQVec++;
      }

    printf( "\n" );
 
    ivptr = HouseFaceList;
    itptr = HouseTFaceList;

        //  Convert vertex points into vertex lines

    for (n = 0; n < HOUSE_FACELIST_MAX; n++ )
      {     // Calculate Vector lengths
        float fDotABC, fDotBCA, fDotCAB;
        uint ia, ib, ic, ta, tb, tc;

        ia = *(ivptr+0);
        ib = *(ivptr+1);
        ic = *(ivptr+2);

        ta = *(itptr+0);
        tb = *(itptr+1);
        tc = *(itptr+2);

        printf( "\nFace #%u    a:%u b:%u c:%u\n", n, ia, ib, ic );
        printf( " A: %f %f %f\n", pQHouse[ia].x, pQHouse[ia].y, pQHouse[ia].z );
        printf( " B: %f %f %f\n", pQHouse[ib].x, pQHouse[ib].y, pQHouse[ib].z );
        printf( " C: %f %f %f\n", pQHouse[ic].x, pQHouse[ic].y, pQHouse[ic].z );

        vmp_QVecSub( pvAB, &pQHouse[ ib ], &pQHouse[ ia ] );  // A to B
        vmp_QVecSub( pvBC, &pQHouse[ ic ], &pQHouse[ ib ] );
        vmp_QVecSub( pvCA, &pQHouse[ ia ], &pQHouse[ ic ] );

        vmp_QVecSub( pvBA, &pQHouse[ ia ], &pQHouse[ ib ] );  // B to A
        vmp_QVecSub( pvCB, &pQHouse[ ib ], &pQHouse[ ic ] );
        vmp_QVecSub( pvAC, &pQHouse[ ic ], &pQHouse[ ia ] );

        printf( " AB: %f %f %f\n", pvAB->x, pvAB->y, pvAB->z );
        printf( " BC: %f %f %f\n", pvBC->x, pvBC->y, pvBC->z );
        printf( " CA: %f %f %f\n", pvCA->x, pvCA->y, pvCA->z );
 
        vmp_QVecNormalize(pvNAB, pvAB);
        vmp_QVecNormalize(pvNBC, pvBC);
        vmp_QVecNormalize(pvNCA, pvCA);

        printf( " norm AB: %f %f %f\n", pvNAB->x, pvNAB->y, pvNAB->z );
        printf( "      BC: %f %f %f\n", pvNBC->x, pvNBC->y, pvNBC->z );
        printf( "      CA: %f %f %f\n", pvNCA->x, pvNCA->y, pvNCA->z );
 
        printf( " Tex #%u    a:%u b:%u c:%u\n", n, ta, tb, tc );
        printf( "  A: %f %f %f\n", pQTex[ta].x, pQTex[ta].y, pQTex[ta].z );
        printf( "  B: %f %f %f\n", pQTex[tb].x, pQTex[tb].y, pQTex[tb].z );
        printf( "  C: %f %f %f\n", pQTex[tc].x, pQTex[tc].y, pQTex[tc].z );

        vmp_QDotProduct(&fDotABC, pvAB, pvBC );
        vmp_QDotProduct(&fDotBCA, pvBC, pvCA );
        vmp_QDotProduct(&fDotCAB, pvCA, pvAB );

        printf( " Dot Product ABC: %f BCA:%f CAB:%f\n", fDotABC, fDotBCA, fDotCAB );
 
        //  NOTE: All vectors (sides of a triangle) all rest in the same plane so they
        //      will all have the same Cross Product (vector perpendicular to the plane)!
        //
        // 3DS Max Counter-clockwise Face orientation

        vmp_QCrossProduct(pvXABC, pvAB, pvAC );
//      vmp_QCrossProduct(pvXBCA, pvBC, pvBA );
//      vmp_QCrossProduct(pvXCAB, pvCA, pvCB );

        printf( "        cross Face: %f %f %f\n", pvXABC->x, pvXABC->y, pvXABC->z );
//      printf( "        cross  BAC: %f %f %f\n", pvXABC->x, pvXABC->y, pvXABC->z );
//      printf( "               CBA: %f %f %f\n", pvXBCA->x, pvXBCA->y, pvXBCA->z );
//      printf( "               ACB: %f %f %f\n", pvXCAB->x, pvXCAB->y, pvXCAB->z );
 
        vmp_QVecNormalize(pvXNABC, pvXABC);
//      vmp_QVecNormalize(pvXNBCA, pvXBCA);
//      vmp_QVecNormalize(pvXNCAB, pvXCAB);

        printf( " Normal cross Face: %f %f %f %s\n", pvXNABC->x, pvXNABC->y, pvXNABC->z, hitA );
//      printf( " Normal cross  BAC: %f %f %f\n", pvXNABC->x, pvXNABC->y, pvXNABC->z );
//      printf( "               CBA: %f %f %f\n", pvXNBCA->x, pvXNBCA->y, pvXNBCA->z );
//      printf( "               ACB: %f %f %f\n", pvXNCAB->x, pvXNCAB->y, pvXNCAB->z );

        printf( "  <<< Force clockwise - Note: Sign is inverted! >>>\n" );

        vmp_QCrossProduct(pvXABC, pvAC, pvAB );
//      vmp_QCrossProduct(pvXBCA, pvBA, pvBC );
//      vmp_QCrossProduct(pvXCAB, pvCB, pvCA );

        printf( "        cross Face: %f %f %f\n", pvXABC->x, pvXABC->y, pvXABC->z );
//      printf( "        cross  CAB: %f %f %f\n", pvXABC->x, pvXABC->y, pvXABC->z );
//      printf( "               ABC: %f %f %f\n", pvXBCA->x, pvXBCA->y, pvXBCA->z );
//      printf( "               BCA: %f %f %f\n", pvXCAB->x, pvXCAB->y, pvXCAB->z );
 
        vmp_QVecNormalize(pvXNABC, pvXABC);
//      vmp_QVecNormalize(pvXNBCA, pvXBCA);
//      vmp_QVecNormalize(pvXNCAB, pvXCAB);

        printf( " Normal cross Face: %f %f %f %s\n", pvXNABC->x, pvXNABC->y, pvXNABC->z, hitB );
//      printf( " Normal cross  CAB: %f %f %f\n", pvXNABC->x, pvXNABC->y, pvXNABC->z );
//      printf( "               ABC: %f %f %f\n", pvXNBCA->x, pvXNBCA->y, pvXNBCA->z );
//      printf( "               BCA: %f %f %f\n", pvXNCAB->x, pvXNCAB->y, pvXNCAB->z );
 
        printf( "\n       ASE Normals: %f %f %f\n", HouseNormals[n].x, HouseNormals[n].y, HouseNormals[n].z );
        ivptr += 3;
        itptr += 3;
      }
}




int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU

    cout << "\nHouse XZY Conversion Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;

    if (!( CPUBITS_MMX & cinfo.nBits ))
      {
        cout << "ERROR! MMX Not supported!\n\n" << flush;
        return -1;
      }

    cout << "Parse as XZY formatted ASE data!" << endl;
    testHouse( iTEST_XZY );

    cout << "________________________________________" << endl;
    cout << "Parse as left-handed XYZ formatted data!" << endl;
    testHouse( iTEST_XYZ_LEFT );

    cout << "Fin!\n\n" << flush;

    return 0;
}

