	TITLE flcX86.asm - FLC (*.flc) Assembly module
	PAGE	53,160

;	Tabs = 8
;
;

	.XLIST

	.686P
	.MMX
	.model flat, C

	include	Prolog32.inc
 

	.LIST



;
;	Flc Color Table
;

FlcRGB	STRUC
	r	db	?
	g	db	?
	b	db	?
FlcRGB	ends



;
;	enum FLC_PACKET - Packet Encoding
;

	FLC_PACKET_SHIFT	=	14

	FLC_PACKET_RUNS		= 0	; Run of Packets
	FLC_PACKET_LASTBYTE 	= 2	; Last Pixel
	FLC_PACKET_SKIPLINE 	= 3	; Skip Lines



extern AsmRunTbl:DWORD

	.data

	ALIGN 	4

boo	dd	0


flcX86	SEGMENT USE32 PUBLIC 'CODE'



;-------------------------------------------------------------------------------------------------
;
;	void flcAsm8Raw( byte *pImg, byte *pZip, uint nStride, uint nWidth, GfxRGB *pRGB );
;
;	Flc - 8-Bit Raw
;	NOTE: This only occurs once optionally as frame #0.
;
	align	16

flcAsm8Raw PROC C PUBLIC USES edi ebx esi pImg:PTR, pZip:PTR, nStride:DWORD, nWidth:DWORD, nHeight:DWORD, pRGB:PTR
	local lcnt:DWORD, sadj:DWORD

	mov	ebx,nStride			; Get Stride of image
	mov	ecx,nWidth			; Get Width of image
	mov	esi,pZip				; Get compression pointer
	sub	ebx,ecx				; ebx=stride adj

	mov	edi,pImg				; Get Destination pointer
	mov	edx,nHeight			; Get Height of image

	mov	eax,ecx
	and	eax,8 -1
	jmp	$CpyTbl[eax*4]

;
;	(n%4)<>0 pixels to copy
;	ecx=width  ebx=strideadjust esi=src edi=dest

;	align	16
;	org	$+14
     org $+8		; Pack functions

$Cpy:	mov	eax,ecx
 	shr	eax,3			; eax = # of 8byte pairs
	jz	$Mini			; Jump if a mini run {1..3}

	mov	sadj,ebx			; Save stride adjustment

;	Copy in 64-bit chunks
;	NOTE: This is just about as fast as MMX

$Cpy1:	mov	ebx,[esi]
	mov	edx,[esi+4]
	mov	[edi],ebx
	mov	[edi+4],edx
	add	esi,8
	add	edi,8
	dec	eax
	jne	$Cpy1

	mov	ah,cl			; Remainders	
	and	ah,000111b		; {1..7}

;	Copy 'n' bytes

$Cpy2:	mov	al,[esi]
	inc	esi
	mov	[edi],al
	inc	edi
	dec	ah			; Any remainders (
	jne	$Cpy2

	mov	eax,ecx			; Width in bytes
	add	edi,sadj			; Adjust to next scanline
 	shr	eax,3			; eax = # of 8byte pairs
	dec	nHeight			; Last scanline
	jnz	$Cpy1			; loop until done

	mov	eax,1			; true
	ret

;
;	Mini-run  ecx={1..7} bytes wide
;

	org	$+12
		; Pack functions

$Mini:	mov	ah,cl

$Mini1:	mov	al,[esi]
	inc	esi
	mov	[edi],al
	inc	edi
	dec	ah
	jne	$Mini1

	add	edi,ebx			; Adjust to next scanline
	dec	edx			; Last scanline
	mov	ah,cl
	jnz	$Mini1			; loop until done

	mov	eax,1			; true
	ret



;
;	Copy in 64-bit chunks
;	NOTE: This is just about as fast as MMX
;
	align	16
	org	$+8
		; Pack functions

$CpyQ:	shr	ecx,3
	mov	sadj,ebx			; Save stride adjustment
	mov	edx,ecx			; eax = # of 8byte pairs

;	Copy scanline

$CpyQ1:	mov	eax,[esi]
	mov	ebx,[esi+4]
	mov	[edi],eax
	mov	[edi+4],ebx
	add	esi,8
	add	edi,8
	dec	edx
	jne	$CpyQ1

	add	edi,sadj			; Start of next scanline

	dec	nHeight
	mov	edx,ecx			; eax = # of 8byte pairs
	jnz	$CpyQ1			; loop until done

	mov	eax,1			; true
	ret


;
;	Copy in 32-bit chunks
;	NOTE: This is just about as fast as MMX
;
;	align	16
    org $+2		; Pack functions

$CpyD:	shr	ecx,2			; 4byte chunks
	mov	edx,ecx			; eax = # of 4byte pairs

;	Copy scanline

$CpyD1:	mov	eax,[esi]
	add	esi,4
	mov	[edi],eax
	add	edi,4
	dec	edx
	jne	$CpyD1			; Loop for all 32bit copies

	add	edi,ebx			; Start of next scanline
	dec	nHeight			; last scanline?
	mov	edx,ecx			; # of 4 byte pairs
	jnz	$CpyD1			; loop until done

	mov	eax,1			; true
	ret


	align	16
$CpyTbl	dd	$CpyQ			; Quad pixel version (n%8)=0
	dd	$Cpy			; (n%4)=1
	dd	$Cpy			; (n%4)=2
	dd	$Cpy			; (n%4)=3
	dd	$CpyD			; DWord pixel (n%4)=0
	dd	$Cpy			; (n%4)=1
	dd	$Cpy			; (n%4)=2
	dd	$Cpy			; (n%4)=3
flcAsm8Raw	endp



;-------------------------------------------------------------------------------------------------
;
;	void flcAsm8Delta( byte *pImg, byte *pZip, uint nStride, uint nWidth, GfxRGB *pRGB  );
;
;	Flc - 8-Bit Delta
;
	align	16
	org	$+14

flcAsm8Delta PROC C PUBLIC USES edi ebx esi pImg:PTR, pZip:PTR, nStride:DWORD, nWidth:DWORD, nHeight:DWORD, pRGB:PTR
	local lcnt:DWORD, tmpebx:DWORD

	mov	esi,pZip
	mov	edi,pImg
	xor	ecx,ecx
	dec	nWidth

	mov	ebx,[esi]		; get Line count
	add	esi,2
	and	ebx,0000ffffh
	mov	lcnt,ebx			; set Line count

	mov	edx,edi

;	Get code

$Nxt:	mov	bx,[esi]			; Get vector data
	add	esi,2

	test	bh,0c0h
	jnz	$Extra			; Jump if FLC_PACKET_LASTBYTE or FLC_PACKET_SKIPLINE

;	Run of 'n' packets    ebx[15:0] ebx[14:0]

	test	bx,bx
	jz	$LstPix			; Only last pixel this scanline

$Runs:	mov	eax,[esi]
	add	esi,4
	mov	ecx,eax			; Get # of pixels to skip
	and	ecx,000000ffh
	add	edi,ecx			; Pixel skip

	mov	cl,ah
	shr	eax,16			; Get 16bit #
	jnc	$Runs5			; c=0 is memcpy, c=1 is memset

;	Run-Length encoded packet (memset)
;	al=0xff..0x80  run of 1..128 double pixels

	inc	cl
	jz	$Runs3a			; only 1 pixel, jump to remainders

	mov	tmpebx,ebx
	shl	eax,16
	mov	ebx,eax
	shr	eax,16
	or	eax,ebx			;eax=double mirrored 16bit
	mov	ebx,tmpebx

$Runs3:	mov	[edi],eax		; Write double pairs! (4pixel writes)
	add	edi,4
	add	cl,2
	jl	$Runs3			; Loop for the pairs
	jnz	$Runs3b			; (No remainder)

;	Remainder?

$Runs3a:	mov	[edi],ax			; Write the pair
	add	edi,2
		
$Runs3b:	dec	ebx
	jnz	$Runs			; Loop until all runs done


$LstPix:	add	edx,nStride
	mov	edi,edx			; Set to start of next scanline
	dec	lcnt			; last scanline?
	jnz	$Nxt			; loop until done

	mov	eax,1			; true
	ret


;
;	Non Run-Length packet (memcpy)
;	ax = #  ecx=pairs

	align	16
	org	$+10

$Runs5:	sub	esi,2
	mov	eax,ecx
	shr	eax,2			; eax = # of 8byte pairs
	jz	$Runs7

;	Copy in 64-bit chunks
;	NOTE: This is just as fast as MMX

	push	edx
	push	ebx

$Runs6:	mov	ebx,[esi]
	mov	edx,[esi+4]
	mov	[edi],ebx
	mov	[edi+4],edx
	add	esi,8
	add	edi,8
	dec	eax
	jne	$Runs6

	pop	ebx
	pop	edx

$Runs7:	and	ecx,3			; Any remainders?
	jz	$Runs8			; Jump if no!

;	Copy in 16-bit chunks

;$Runs7:  (Move up due to an undocumented length of 0 is in art!)
$Runs7a:	mov	eax,[esi]
	add	esi,2
	mov	[edi],ax
	add	edi,2
	dec	ecx
	jne	$Runs7a

$Runs8:	dec	ebx
	jnz	$Runs			; Loop until all runs done

	add	edx,nStride
	mov	edi,edx			; Set to start of next scanline
	dec	lcnt			; last scanline?
	jnz	$Nxt			; loop until done

	mov	eax,1			; true
	ret



	align	16
;	bx = # & info

$Extra:	jpe	$Skip			; (parity even) Jump if 2 bits set

;	Write last byte of an (odd width) scanline  8000h
;	bl = pixel

$Last:	mov	eax,nWidth		; Note: nWidth is really (nWidth-1)
	mov	[edi+eax],bl		; Set last pixel
	jmp	$Nxt



;
;	Skip 'n' lines  c000h
;	bx = inverted lineskip

	align	16
$Skip:	mov	eax,10000h
	sub	eax,ebx
	mov	ebx,nStride
	mul	ebx

	add	edi,eax			; skip them
	mov	edx,edi			; save new dst pointer

;
;	Alternate $Nxt
;

$Nxt2:	mov	bx,[esi]			; Get vector data
	add	esi,2

	test	bh,0c0h
	jnz	$Extra			; Jump if FLC_PACKET_LASTBYTE or FLC_PACKET_SKIPLINE

;	Run of 'n' packets    ebx[15:0] ebx[14:0]

	test	bx,bx
	jnz	$Runs

	jmp	$LstPix			; Only last pixel this scanline

flcAsm8Delta	endp




;-------------------------------------------------------------------------------------------------
;
;	void flcAsm8Rle( byte *pImg, byte *pZip, uint nStride, uint nWidth, GfxRGB *pRGB );
;
;	Flc - 8-Bit Run Length Encoding
;	NOTE: This only occurs once optionally as frame #0.
;
; FIXME - we can make it go even faster by going to (n mod 4) vectors or just 256 vectors!
	align	16

flcAsm8Rle PROC C PUBLIC USES edi ebx esi pImg:PTR, pZip:PTR, nStride:DWORD, nWidth:DWORD, nHeight:DWORD, pRGB:PTR
	local sadj:DWORD

	mov	ebx,nStride
	mov	ecx,nWidth
	sub	ebx,ecx			; calculate stride adjustment
	mov	edi,pImg
	mov	esi,pZip
	xor	edx,edx
	mov	sadj,ebx			; set stride adjustment
	mov	ebx,nHeight		; Image height

;	New scanline

$L1:	add	ecx,edi			; pEnd = pImg + nWidth
	inc	esi			; toss byte

;	New pixel run
;	edx=0

$L2:	mov	edx,[esi]			;run
	inc	esi
	and	edx,000000ffh
	jz	$L2			; Run of 0! (Not spec may exist!)

	mov	eax,edx
	and	eax,10000011b
	rol	al,1			; rotate DB7 into DB0 for a table index
	jmp	$RleTbl[eax*4]		


	align	16

	;	memcpy()
	;	dl=-run

$MCpy:	mov	al,[esi]			; Get pixel color
	inc	esi
	mov	[edi],al			; Set pixel color
	inc	edi
	inc	dl
	jne	$MCpy			; Loop for run

	cmp	edi,ecx			; end of scanline?
	jb	$L2			; Jump if processing another pixel run

	add	edi,sadj			; stride adjust
	mov	ecx,nWidth
	dec	ebx			; All scanlines?
	jnz	$L1			; Jump if processing another scanline

	mov	eax,1			; true
	ret



	align	16
	;	32-bit memcpy()  (n%4)=0
	;	dl=-run

$MCpyD:	mov	eax,[esi]		; Get pixel color
	add	esi,4
	mov	[edi],eax		; Set pixel color
	add	edi,4
	add	dl,4
	jne	$MCpyD			; Loop for run

	cmp	edi,ecx			; end of scanline?
	jb	$L2			; Jump if processing another pixel run

	add	edi,sadj			; stride adjust
	mov	ecx,nWidth
	dec	ebx			; All scanlines?
	jnz	$L1			; Jump if processing another scanline

	mov	eax,1			; true			  


	;	memset()
	;	
	align	16


$MSet:	mov	al,[esi]			; Get pixel color
	
$MSet1:	mov	[edi],al			; Save pixel
	inc	edi
	dec	dl
	jne	$MSet1			; Loop for run

	inc	esi
	cmp	edi,ecx			; end of scanline?
	jb	$L2			; Jump if processing another pixel run

	add	edi,sadj			; stride adjust
	mov	ecx,nWidth
	dec	ebx			; All scanlines?
	jnz	$L1			; Jump if processing another scanline

	mov	eax,1			; true
	ret



;
;	32-bit memset
;

	align	16

$MSetD:	mov	al,[esi]			; Get pixel color
	
$MSetD1:	mov	[edi],al			; Save pixel
	inc	edi
	dec	dl
	jne	$MSetD1			; Loop for run

	inc	esi
	cmp	edi,ecx			; end of scanline?
	jb	$L2

	add	edi,sadj			; stride adjust
	mov	ecx,nWidth
	dec	ebx			; All scanlines?
	jnz	$L1

	mov	eax,1			; true
	ret


$RleTbl	dd	offset $MSetD		; 00 0	; (n%4)=0 set
	dd	offset $MCpyD		; 00 1	; (n%4)=0 cpy
	dd	offset $MSet		; 01 0	; (n%4)=1 set
	dd	offset $MCpy		; 01 1	; (n%4)=1 cpy
	dd	offset $MSet		; 10 0	; (n%4)=2 set
	dd	offset $MCpy		; 10 1	; (n%4)=2 cpy
	dd	offset $MSet		; 11 0	; (n%4)=3 set
	dd	offset $MCpy		; 11 1	; (n%4)=3 cpy

flcAsm8Rle	endp




flcX86	ends
	end

