  
//
//  Matrix3D.h - 3D Math Matrix Function
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to accomodate the book.
//
//      This module is designed for fast 3D math
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include    "KariType.h"                                // Kari Games Master Definitions
#include    "vmp3D.h"



#ifndef __VTrig3D_H
#define __VTrig3D_H

#ifdef __cplusplus
extern "C" {
#endif



void vmp_FSinCosGeneric( float * const pfSin, float * const pfCos, float fRads );
void vmp_FSinCosAsmFPU(  float * const pfSin, float * const pfCos, float fRads );

#define vmp_FSinCos vmp_FSinCosGeneric




//                            | 1 0 0 0 |
//  Matrix - Set Identity     | 0 1 0 0 |
//                            | 0 0 1 0 |
//                            | 0 0 0 1 |
typedef void (*vmp_MatrixSetIdentityProc)( vmp3DMatrix dMx );
extern vmp_MatrixSetIdentityProc vmp_MatrixSetIdentity;


//
//  Matrix - Set Scaling
//

typedef void (*vmp_MatrixSetScalingProc)( vmp3DMatrix dMx, float fTx, float fTy, float fTz );
extern vmp_MatrixSetScalingProc vmp_MatrixSetScaling;

//
//  Matrix - Transpose
//

typedef void (*vmp_MatrixTransposeProc)( vmp3DMatrix dMx, const vmp3DMatrix aMx );
extern vmp_MatrixTransposeProc vmp_MatrixTranspose;

//
//  Matrix - Set Translate
//

typedef void (*vmp_MatrixSetTranslateProc)( vmp3DMatrix dMx, float fTx, float fTy, float fTz );
extern vmp_MatrixSetTranslateProc vmp_MatrixSetTranslate;

//
//  Matrix - Set Rotation
//
typedef void (*vmp_MatrixSetRotateProc)( vmp3DMatrix dMx, float fRads );
extern vmp_MatrixSetRotateProc vmp_MatrixSetRotateX;
extern vmp_MatrixSetRotateProc vmp_MatrixSetRotateY;
extern vmp_MatrixSetRotateProc vmp_MatrixSetRotateZ;



typedef void (*vmp_MatrixCopyProc)( vmp3DMatrix dMx, const vmp3DMatrix aMx );
extern vmp_MatrixCopyProc vmp_MatrixCopy;

typedef void (*vmp_MatrixAddProc)( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
extern vmp_MatrixAddProc vmp_MatrixAdd;

typedef void (*vmp_MatrixMulProc)( vmp3DMatrix dMx, const vmp3DMatrix aMx, const vmp3DMatrix bMx );
extern vmp_MatrixMulProc vmp_MatrixMul;



void vmp_MatrixTranslate( vmp3DMatrix dMx, const vmp3DMatrix aMx, float fTx, float fTy, float fTz );

void vmp_MatrixRotateX( vmp3DMatrix dMx, const vmp3DMatrix aMx, float fRads );
void vmp_MatrixRotateY( vmp3DMatrix dMx, const vmp3DMatrix aMx, float fRads );
void vmp_MatrixRotateZ( vmp3DMatrix dMx, const vmp3DMatrix aMx, float fRads );


#ifdef __cplusplus
}
#endif

#endif  //__Matrix3D_H


