
//
//  Bench.cpp - YUV12 to RGB
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 25 Feb, 2002 - James Leiterman
//
//  Last Modification
//
//
//  stab=4
//



#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include <iostream.h>
#include <assert.h>
#include <string.h>

    //  Book Development Library Modules
    
#include "CpuAsm.h"                                     // CPU Module
#include "ram.h"
#include "vmp3D.h"
#include "yuv12.h"
#include "yuv12Ex.h"
#include "yuv12Data.h"                                  // Test Data


typedef enum
{
    iTEST_YUV12_RGB32

} iTEST;



#define IMG_STRIDE  ALIGN16(IMG_WIDTH*4)
#define IMG_SIZE    (IMG_HEIGHT * IMG_STRIDE)



static bool IsBufferClose( const uint8 *bt, const uint8 *bu, uint nImgSize, uint Diff )
{
    ASSERT_PTR(bt);
    ASSERT_PTR(bu);
    ASSERT_ZERO(nImgSize);

    do {
        if ( *bt <= *bu )
          {
            if ( ((uint)(*bu) - (uint)(*bt)) > Diff )
              {
                return false;
              }
          }
        else if ( ((uint)(*bt) - (uint)(*bu)) > Diff )
          {
            return false;
          }

        bt++;
        bu++;
    } while (--nImgSize);

    return true;
}


//
//  Test 
//
static void test_yuv( iTEST iFunc )
{
    uint    nImgWidth, nImgHeight, nStrideUV, nStrideY, nImgSize;
    int     iImgStride;
    uint8   *pBaseY, *pBaseU, *pBaseV, *bt, *bu;

    if (!ramGetClr( &pBaseY, (Y_STRIDE * IMG_HEIGHT) + ((UV_STRIDE * (IMG_HEIGHT/2))*2) ) )
      {
        return;
      }

        //  Image parameters

    pBaseU = pBaseY + (Y_STRIDE * IMG_HEIGHT);
    pBaseV = pBaseU + (UV_STRIDE * (IMG_HEIGHT>>1));

        //  YUV REQUIRED to be aligned!

    ASSERT_PTR16(pBaseY);
    ASSERT_PTR16(pBaseU);
    ASSERT_PTR16(pBaseV);

    memcpy( pBaseY, YTblTestData,  Y_STRIDE *  IMG_HEIGHT);
    memcpy( pBaseU, UTblTestData, UV_STRIDE * (IMG_HEIGHT>>1));
    memcpy( pBaseV, VTblTestData, UV_STRIDE * (IMG_HEIGHT>>1));

    nStrideY = Y_STRIDE;
    nStrideUV = UV_STRIDE;
    nImgWidth = IMG_WIDTH;
    nImgHeight = IMG_HEIGHT;
    iImgStride = IMG_STRIDE;
    nImgSize = IMG_SIZE;

        //  YUV AND Image Frame Buffer REQUIRED to be 16 byte aligned!
        
    if (!ramGetClr( &bt, 2*IMG_SIZE ))
      {
        ramRelease( &pBaseY );
        return;
      }

    bu = bt + IMG_SIZE;

    do
      {
        memset( bt, 0x55, nImgSize );                   // Preset Primary Frame buffer with non-zero value
        memset( bu, 0xaa, nImgSize );                   // Set Secondary    "   "   "

        switch (iFunc)
          {
            case iTEST_YUV12_RGB32:
                vmp_yuv12toRGB32Generic( bt, nImgWidth, nImgHeight, iImgStride,
                                         pBaseY, nStrideY, 
                                         pBaseU, pBaseV, nStrideUV );
                vmp_yuv12toRGB32(        bu, nImgWidth, nImgHeight, iImgStride,
                                         pBaseY, nStrideY, 
                                         pBaseU, pBaseV, nStrideUV );
                break;
          }

            //  NOTE: They won't be exactly the same but should be close
            //      due to the pre f(n<<3)>>16 instead of f(n)>>13

        if ( !IsBufferClose( bt, bu, nImgSize, 3 ) )
          {
            cout << "Mismatch " << endl;
          }
      } while(0);

    ramRelease( &bt );
    ramRelease( &pBaseY );
}



int main( int argc, char *argv[] )
{
    CpuInfo cinfo;
    char szBuf[ CPU_SZBUF_MAX ];

    CpuDetect( &cinfo );                                // Detect CPU
    ramSetup( &cinfo );
    
    cout << "\nYUV to RGB Test\n\n";
    cout << CpuInfoStr( szBuf, &cinfo ) << endl;
    
    yuv12Setup( &cinfo );                               // Setup our YUV to RGB

    cout << "Testing YUV12 to RGB32" << endl;
    test_yuv( iTEST_YUV12_RGB32 );

    cout << "Fin\n\n" << flush;

    return 0;
}

