
//
//  vmp3D.cpp - 3D Vector
//
//  This document contains file that you may use at your own risk in your own programs.
//
//  This module is designed to work with the exercises for this book.
//
//  Created - 3 Oct 2001 - James Leiterman
//
//  Last Modification
//
//  stab=4
//


#include "KariType.h"                                   // Master Definitions

    //  3rd Party Libraries
    
#include    <assert.h>
#include    <iostream.h>

    //  Book Development Library Modules

#include    "CpuAsm.h"                                  // CPU
#include    "vmp3D.h"                                   // 3D Math
#include    "vmp3DEx.h"





#ifdef VMP_ADVANCED
vmp_IsVEqualProc        vmp_IsVEqual;
vmp_IsQVEqualProc       vmp_IsQVEqual;
vmp_IsMatrixEqualProc   vmp_IsMatrixEqual;
#endif



//
//  Compare two floating point values
//

bool vmp_IsFEqual(float fA, float fB, float fPrec)
{
    if (fA == fB)
      {
        return true;
      }

  return (((fA-fPrec)<=fB) && (fB<=(fA+fPrec)));
}



//
//  Compare two floating point values
//

bool vmp_IsDEqual(double fA, double fB, double fPrec)
{
  if (fA == fB) return true;

  return (((fA-fPrec)<=fB) && (fB<=(fA+fPrec)));
}



//
//  Compare two Vectors
//

bool vmp_IsVEqualGeneric( const vmp3DVector * const pvA, const vmp3DVector * const pvB, float fPrec )
{
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    
    if (   !vmp_IsFEqual(pvA->x, pvB->x, fPrec) 
        || !vmp_IsFEqual(pvA->y, pvB->y, fPrec) 
        || !vmp_IsFEqual(pvA->z, pvB->z, fPrec))
      {
        return false;
      }

    return true;
}                       


//
//  Compare two (Quad) Vectors
//

bool vmp_IsQVEqualGeneric( const vmp3DQVector * const pvA, const vmp3DQVector * const pvB, float fPrec )
{
    ASSERT_PTR4(pvA);
    ASSERT_PTR4(pvB);
    
    if (   !vmp_IsFEqual(pvA->x, pvB->x, fPrec) 
        || !vmp_IsFEqual(pvA->y, pvB->y, fPrec) 
        || !vmp_IsFEqual(pvA->z, pvB->z, fPrec) 
        || !vmp_IsFEqual(pvA->w, pvB->w, fPrec))
      {
        return false;
      }

    return true;
}                                               



//
//  Compare two 4x4 Matrix
//

bool vmp_IsMatrixEqualGeneric( vmp3DMatrix aMx, vmp3DMatrix bMx, float fPrecision)
{
    uint u;
    float *pfA, *pfB;
    
    ASSERT_PTR4(aMx);
    ASSERT_PTR4(bMx);
    
            // Slow but understandable!

    pfA = (float *) aMx;
    pfB = (float *) bMx;
    
    u = 16;
    while (u--)
      {
        if (!vmp_IsFEqual( *pfA++, *pfB++, fPrecision ))
          {
            return false;
          }
      }

    return true;
}






